@@ -12,18 +12,73 @@ class SmarterSender:
         self.delegate = delegate
 
     def send_message(self, message: SmarterMessage, port: str) -> Optional[SmarterMessage]:
+        """
+        Takes in a message from a python code component and sends it to its output port.
+        :param message: A SmarterMessage to be sent through an output port
+        :param port: The output port to be
+        :return:
+        """
         return self.delegate(message, port)
 
     def set_data(self, pattern: str, data: Any) -> None:
-        '''
+        """
         Takes in JSON serializable data and sets it to a specific front-end GUI component.
         :param pattern: The front-end GUI pattern to set the data to
         :param data: The data to be sent to the GUI. It needs to match the format the pattern expects the data in.
                         Example a chart expects a table-like format, while a textField expects some text.
-        :return:
-        '''
-        message = SmarterMessage({"action": "setData", "args": {"pattern": pattern, "data": data}})
-        self.send_message(message, '')
+        :return: None
+        """
+        message = SmarterMessage({"action": "setData",
+                                  "args": {"pattern": pattern, "data": data}})
+        self.send_message(message=message, port='#gui')
+
+    def clear_data(self, pattern: str) -> None:
+        """
+        Clears any data associated with a specific pattern in the GUI components
+        :param pattern: The front-end GUI pattern to set the data to
+        :return: None
+        """
+        message = SmarterMessage({"action": "setData",
+                                  "args": {"pattern": pattern, "data": None}})
+        self.send_message(message=message, port='#gui')
+
+    def append_data(self, pattern: str, data: Any, limit: int) -> None:
+        """
+        Appends new data to previously sent data to a specific pattern.
+        :param pattern: The front-end GUI pattern to append the data to.
+        :param data: The data to be appended to a GUI component's previous data.
+        :param limit: If the data's total size is bigger than limit, then a sliding window will be implemented to hold
+            the latest added elements
+        :return: None
+        """
+        message = SmarterMessage({"action": "setData",
+                                  "args": {"pattern": pattern, "data": data},
+                                  "options": {"append": True, "limitLength": limit}})
+        self.send_message(message=message, port='#gui')
+
+    def prepend_data(self, pattern: str, data: Any, limit: int) -> None:
+        """
+        Prepends new data to previously sent data to a specific pattern.
+        :param pattern: The front-end GUI pattern to append the data to.
+        :param data: The data to be prepended to a GUI component's previous data.
+        :param limit: If the data's total size is bigger than limit, then a sliding window will be implemented to hold
+            the latest added elements
+        :return: None
+        """
+        message = SmarterMessage({"action": "setData",
+                                  "args": {"pattern": pattern, "data": data},
+                                  "options": {"prepend": True, "limitLength": limit}})
+        self.send_message(message=message, port='#gui')
+
+    def reply_back(self, message: SmarterMessage) -> None:
+        """
+        Takes in a message to reply back to front-end REST/Websocket topics. An equivalent  to 'return' with a message.
+        Uses built-in port #action to identify the front-end topic
+        :param message: A SmarterMessage JSON Serializable
+        :return: None
+        """
+
+        self.send_message(message=message, port='#action')
 
 
 class _Smarter_SignatureCheckerMeta(ABCMeta):
